/* This code is copyrighted (c) 1997 by Marshall Ramsey */
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.zip.*;
import java.applet.*;
import java.net.*;

public class KanjiFlash extends Frame implements ActionListener, ItemListener {
  static final String copyright = "(c) 1997 by Marshall Ramsey";
  static final boolean debug = false;
  static final String version = "v1.2";
  boolean registered;
  Applet app;
  TextCanvas kanji;
  TextCanvas yomi;
  TextCanvas meaning;
  TextField guess;
  KanaInput kanaInput;
  int fontSize;
  Label yLabel;
  Label mLabel;
  Label kLabel;
  Label gResultsLabel;
  Label gLabel;
  Label results;
  Label status;
  Scribble scribble;
  Button show;
  Button right;
  Button wrong;
  Lesson lesson;
  LessonItem curItem;
  String fileName;
  String lessonName;
  int numPresented;
  boolean useBitMaps;

  LessonChooser chooser;

  // declare menus
  Menu fileMenu;
  Menu optionMenu;
  Menu helpMenu;

  // declare menu items
  MenuItem openMI;
  MenuItem multiOpenMI;
  MenuItem exitMI;

  MenuItem restartMI;
  MenuItem clearHistoryMI;
  MenuItem prefMI;

  MenuItem aboutMI;

  // declare other components
  Checkbox yomiShowing;
  Checkbox meaningShowing;
  Checkbox kanjiShowing;

  Choice lessonModeChoice;
  Choice fontSizeChoice;

  // define preferences dialog components
  Dialog prefDialog;
  Button prefOK;

  // bitmap fonts that may be used
  BitMapFont largeFont;
  BitMapFont smallFont;

  public KanjiFlash(Applet app, boolean registered, boolean useBitMaps) {
    super("KanjiFlash " + version + //(registered?" (registered) ":" (unregistered) ") +
	  " " + copyright);
    this.registered = registered;
    this.app = app;
    this.useBitMaps = useBitMaps;
    initialize();
  }

  public void initialize() {
    numPresented = 0;
    fontSize = 0;
    MenuBar mb = new MenuBar();
    fileMenu = new Menu("File");
    openMI = new MenuItem("Open...");
    openMI.addActionListener(this);
    exitMI = new MenuItem("Exit");
    exitMI.addActionListener(this);
    fileMenu.add(openMI);
    multiOpenMI = new MenuItem("Multi-Open...");
    if (app == null) {
      multiOpenMI.addActionListener(this);
      fileMenu.add(multiOpenMI);
    }
     if (!registered)
       multiOpenMI.setEnabled(false);
    fileMenu.addSeparator();
    fileMenu.add(exitMI);
    mb.add(fileMenu);
    optionMenu = new Menu("Option");
    restartMI = new MenuItem("Restart");
    restartMI.setEnabled(false);
    restartMI.addActionListener(this);
    prefMI = new MenuItem("Preferences...");
    prefMI.addActionListener(this);

    yomiShowing = new Checkbox("Show Reading", true);
    yomiShowing.addItemListener(this);
    meaningShowing = new Checkbox("Show Meaning", true);
    meaningShowing.addItemListener(this);
    kanjiShowing = new Checkbox("Show Kanji", false);
    kanjiShowing.addItemListener(this);

    clearHistoryMI = new MenuItem("Clear Lesson(s) History");
    clearHistoryMI.addActionListener(this);

    clearHistoryMI.setEnabled(false); // enable only if a lesson is loaded, app is null, and
                                      // the user has registered (done later)

    //create lesson mode drop-down list
    lessonModeChoice = new Choice();
    lessonModeChoice.add("Normal");
    if (registered) {
      lessonModeChoice.add("Quiz 25");
      lessonModeChoice.add("Quiz 50");
      if (app == null)
	lessonModeChoice.add("Review");
    }
    lessonModeChoice.addItemListener(this);

    //create font size drop-down list
    fontSizeChoice = new Choice();
    fontSizeChoice.add("Fit-to-Size");
    fontSizeChoice.add("12pt");
    fontSizeChoice.add("18pt");
    fontSizeChoice.add("24pt");
    fontSizeChoice.add("36pt");
    fontSizeChoice.add("48pt");
    fontSizeChoice.addItemListener(this);

    if (useBitMaps) // cannot change bitmap font size
      fontSizeChoice.setEnabled(false);

    optionMenu.add(restartMI);
    optionMenu.addSeparator();
    optionMenu.add(clearHistoryMI);
    optionMenu.addSeparator();
    optionMenu.add(prefMI);

    mb.add(optionMenu);
    helpMenu = new Menu("Help");
    aboutMI = new MenuItem("About...");
    aboutMI.addActionListener(this);
    helpMenu.add(aboutMI);
    mb.setHelpMenu(helpMenu);
    setMenuBar(mb);
    
    loadTextCanvases();

    scribble = new Scribble("Kanji Writing Pad:");
    scribble.setText("You can draw kanji here.");
    mLabel = new Label("Meaning:");
    yLabel = new Label("Reading:");
    kLabel = new Label("Kanji:");
    results = new Label("Results: 0/0", Label.LEFT);
    status = new Label("Status: No lesson loaded", Label.LEFT);
    
    guess = new TextField("You can type the meaning here.");
    guess.setForeground(Color.blue);
    guess.setFont(new Font("sansserif", Font.PLAIN, 16));
    gLabel = new Label("Meaning?:", Label.RIGHT);
    gResultsLabel = new Label("", Label.CENTER);

    if (useBitMaps)
      kanaInput = new KanaInput(new BitMapTextCanvas(smallFont, 2, 10));
    else
      kanaInput = new KanaInput();
    kanaInput.setFieldsForeground(Color.blue);
    kanaInput.setRomaji("You can enter the reading here.");

    show = new Button("Show");
    show.setEnabled(false);
    show.addActionListener(this);
    right = new Button("Right");
    right.setEnabled(false);
    right.addActionListener(this);
    wrong = new Button("Wrong");
    wrong.setEnabled(false);
    wrong.addActionListener(this);
    setLayout(new GridBagLayout());

    Panel lessonPanel = new BorderedPanel(new GridBagLayout());
    Tools.constrain(lessonPanel, yLabel, 0, 0, 1, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.SOUTHWEST,
		    0.5, 0.0, 5, 5, 5, 5);
    Tools.constrain(lessonPanel, yomi, 0, 1, 2, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
		    1.0, 0.1, 5, 5, 5, 5);
    Tools.constrain(lessonPanel, mLabel, 0, 2, 1, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.SOUTHWEST,
		    0.5, 0.0, 5, 5, 5, 5);
    Tools.constrain(lessonPanel, meaning, 0, 3, 2, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.NORTHWEST,
		    1.0, 0.1, 5, 5, 5, 5);
    Tools.constrain(lessonPanel, kLabel, 0, 4, 1, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.SOUTHWEST,
		    0.5, 0.0, 5, 5, 5, 5);
    Tools.constrain(lessonPanel, kanji, 0, 5, 2, 7,
		    GridBagConstraints.BOTH, GridBagConstraints.CENTER,
		    1.0, 0.8, 5, 5, 5, 5);

    Panel buttonPanel = new Panel(new GridLayout(1,2,2,2));
    Panel bp1 = new Panel(new GridLayout(1,2,2,2));
    bp1.add(right);
    bp1.add(wrong);
    buttonPanel.add(show);
    buttonPanel.add(bp1);

    Panel gPanel = new BorderedPanel(new GridBagLayout());
    Tools.constrain(gPanel, gLabel, 0, 0, 1, 1,
		    GridBagConstraints.NONE, GridBagConstraints.EAST,
		    0.0, 0.0, 5, 5, 5, 5);
    Tools.constrain(gPanel, guess, 1, 0, 4, 1,
		    GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST,
		    1.0, 0.0, 5, 5, 5, 5);
    Tools.constrain(gPanel, kanaInput.kLabel, 0, 1, 1, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.EAST,
		    0.0, 1.0, 5, 5, 5, 5);
    Tools.constrain(gPanel, kanaInput.kana, 1, 1, 4, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.WEST,
		    1.0, 1.0, 5, 5, 5, 5);
    Tools.constrain(gPanel, kanaInput.rLabel, 0, 2, 1, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.EAST,
		    0.0, 0.0, 5, 5, 5, 5);
    Tools.constrain(gPanel, kanaInput.romaji, 1, 2, 4, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.WEST,
		    1.0, 0.0, 5, 5, 5, 5);

    Panel answerPanel = new Panel(new GridBagLayout());
    Tools.constrain(answerPanel, gPanel, 0, 0, 1, 3,
		    GridBagConstraints.BOTH, GridBagConstraints.CENTER,
		    1.0, 0.0, 5, 5, 5, 5);
    Tools.constrain(answerPanel, scribble, 0, 3, 1, 9,
		    GridBagConstraints.BOTH, GridBagConstraints.CENTER,
		    1.0, 1.0, 5, 5, 5, 5);

    Panel resultsPanel = new Panel(new GridBagLayout());
    Tools.constrain(resultsPanel, results, 0, 0, 1, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.EAST,
		    0.5, 0.0, 5, 5, 5, 5);
    Tools.constrain(resultsPanel, status, 2, 0, 1, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.EAST,
		    0.5, 0.0, 5, 5, 5, 5);

    Panel p1 = new Panel(new GridLayout(1,2,2,2));
    p1.add(lessonPanel);
    p1.add(answerPanel);
    Tools.constrain(this, p1, 0, 0, 2, 6,
		    GridBagConstraints.BOTH, GridBagConstraints.CENTER,
		    1.0, 1.0, 5, 5, 5, 5);
    Tools.constrain(this, buttonPanel, 0, 6, 2, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.CENTER,
		    1.0, 0.0, 5, 5, 5, 5);
    Tools.constrain(this, resultsPanel, 0, 7, 2, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.SOUTH,
		    1.0, 0.0, 5, 5, 5, 5);

    prefOK = new Button("OK");
    prefOK.addActionListener(this);

    setBackground(Color.lightGray);
    setSize(775, 575);
    setVisible(true);
  }

  public void actionPerformed(ActionEvent e) {
    Object target = e.getSource();
    if (target instanceof Button) {
      if (target == show)
	showKanji();
      else if (target == right)
	selectedRight();
      else if (target == wrong)
	selectedWrong();
      else if (target == prefOK)
	selectedPreferencesOK();
      else if (e.getActionCommand().equals("OK"))
	selectedLessonChooserOK();
    }
    else if (target instanceof MenuItem) {
      if (target == openMI) {
	if (app != null)
	  selectedOpen();
	else
	  selectedOpenWithLocalIO();
      }
      if (target == multiOpenMI) {
	selectedMultiOpen();
      }
      else if (target == exitMI) {
	selectedExit();
      }
      else if (target == restartMI) {
	selectedRestart();
      }
      else if (target == clearHistoryMI) {
	selectedClearHistory();
      }
      else if (target == prefMI) {
	selectedPreferences();
      }
      else if (target == aboutMI) {
	selectedAbout();
      }
    }
  }
  
  public void itemStateChanged(ItemEvent e) {
    Object target = e.getSource();
    if (target instanceof Choice) {
      if (target == lessonModeChoice) {
	switch(lessonModeChoice.getSelectedIndex()) {
	case 0  : selectedNormalLesson(); break;
	case 1  : selectedQuiz(25); break;
	case 2  : selectedQuiz(50); break;
	case 3  : selectedReviewMode(); break;
	default : selectedNormalLesson();
	}
      }
      else if (target == fontSizeChoice) {
	switch(fontSizeChoice.getSelectedIndex()) {
	case 0  : setKanjiMaxSize(0); break;
	case 1  : setKanjiMaxSize(12); break;
	case 3  : setKanjiMaxSize(18); break;
	case 4  : setKanjiMaxSize(24); break;
	case 5  : setKanjiMaxSize(36); break;
	case 6  : setKanjiMaxSize(48); break;
	default : setKanjiMaxSize(0);
	}
      }
    }
    else if (target instanceof Checkbox) {
      if (target == yomiShowing) {
	selectedYomiShowing();
      }
      else if (target == meaningShowing) {
	selectedMeaningShowing();
      }
      else if (target == kanjiShowing) {
	selectedKanjiShowing();
      }
    }
  }

  public void commonPostOpen() {
    guess.setForeground(Color.black);
    //gResultsLabel.setText("");
    //gResultsLabel.setBackground(Color.lightGray);
    kanaInput.setFieldsForeground(Color.black);
    kanji.setForeground(Color.black);
    right.setBackground(getBackground());
    wrong.setBackground(getBackground());
    restartMI.setEnabled(true);
    if (registered) {
      //restartMI.setEnabled(true);
      if (app == null) {
	clearHistoryMI.setEnabled(true);
      }
    }

    if (lessonModeChoice.getSelectedIndex()==1)
      lesson.randomSubset(25);
    else if (lessonModeChoice.getSelectedIndex()==2)
      lesson.randomSubset(50);
    
    if (lessonModeChoice.getSelectedIndex()==3)
      lesson.setReviewMode(true);
    else
      lesson.setReviewMode(false);

    if (app == null && lessonModeChoice.getSelectedIndex() == 3)
      results.setText("Results: 0/0");
    else
      results.setText("Results: 0/0 (" + lesson.getCurSize() + ")");
  }

  public void nextLessonItem() {
    if (debug)
      System.out.println("nextLessonItem called");
    // This program has had out-of-memory errors so frequently
    // garbage collect
    System.gc();
    if (lesson != null && (curItem = lesson.nextItem()) != null
	&& curItem.isValid()) {
      if (!kanjiShowing.getState()) // if kanji should be hidden
	kanji.setVisible(false);
      kanji.setText(curItem.getKanji());
      if (!yomiShowing.getState()) // if yomi should be hidden
	yomi.setVisible(false);
      yomi.setText(curItem.getYomi());
      if (!meaningShowing.getState()) // if meaning should be hidden
	meaning.setVisible(false);
      meaning.setText(curItem.getMeaning());
      scribble.clear();
      guess.setText("");
      kanaInput.clear();
      show.setBackground(Color.blue);
      show.setEnabled(true);
      right.setEnabled(false);
      wrong.setEnabled(false);
    }
    else {
      show.setBackground(Color.lightGray);
      show.setEnabled(false);
      right.setEnabled(false);
      wrong.setEnabled(false);
      status.setText("Status: " + lessonName + " completed (" + lesson.getSize() + ")");
    }
  }

  public void showKanji() {
    if (debug)
      System.out.println("showKanji called");
    if (!kanjiShowing.getState()) // if kanji is hidden
      kanji.setVisible(true);
    if (!meaningShowing.getState()) // if meaning is hidden
      meaning.setVisible(true);
    if (!yomiShowing.getState()) // if yomi is hidden
      yomi.setVisible(true);
    show.setEnabled(false);
    show.setBackground(Color.lightGray);
    if (!guess.getText().equals("")) {
      // if the user tried to guess the meaning check it
      if (curItem.meaningRight(guess.getText())) {
	//gResultsLabel.setForeground(Color.black);
	//gResultsLabel.setBackground(Color.green);
	//gResultsLabel.setText("Correct");
	right.setEnabled(true);
	right.setBackground(Color.green);
      }
      else {
	//gResultsLabel.setForeground(Color.black);
	//gResultsLabel.setBackground(Color.red);
	//gResultsLabel.setText("Incorrect");
	wrong.setEnabled(true);
	wrong.setBackground(Color.red);
      }
    }
    else if (!kanaInput.getKana().equals("")) {
      // if the user tried to guess the reading check it
      if (curItem.yomiRight(kanaInput.getKana())) {
	//gResultsLabel.setForeground(Color.black);
	//gResultsLabel.setBackground(Color.green);
	//gResultsLabel.setText("Correct");
	right.setEnabled(true);
	right.setBackground(Color.green);
      }
      else {
	//gResultsLabel.setForeground(Color.black);
	//gResultsLabel.setBackground(Color.red);
	//gResultsLabel.setText("Incorrect");
	wrong.setEnabled(true);
	wrong.setBackground(Color.red);
      }
    }
    else {
      right.setEnabled(true);
      right.setBackground(Color.green);
      wrong.setEnabled(true);
      wrong.setBackground(Color.red);
    }
  }

  public void selectedRight() {
    lesson.markRight(curItem);
    if (!registered || app != null || lessonModeChoice.getSelectedIndex() != 3) {
      results.setText("Results: " + lesson.getNumRight() + "/" +
		      lesson.getNumTotal() + "  " +
		      (int)(lesson.getPercent()*100.0) + "% (" + lesson.getCurSize() + ")");
    }
    else {
      results.setText("Results: " + lesson.getNumRight() + "/" +
		      lesson.getNumTotal() + "  " +
		      (int)(lesson.getPercent()*100.0) + "%");
    }
    numPresented++;
    right.setBackground(Color.lightGray);
    wrong.setBackground(Color.lightGray);
    nextLessonItem();
  }

  public void selectedWrong() {
    lesson.markWrong(curItem);
    if (!registered || app != null || lessonModeChoice.getSelectedIndex() != 3) {
      results.setText("Results: " + lesson.getNumRight() + "/" +
		      lesson.getNumTotal() + "  " +
		      (int)(lesson.getPercent()*100.0) + "% (" + lesson.getCurSize() + ")");
    }
    else {
      results.setText("Results: " + lesson.getNumRight() + "/" +
		      lesson.getNumTotal() + "  " +
		      (int)(lesson.getPercent()*100.0) + "%");
    }
    numPresented++;
    right.setBackground(Color.lightGray);
    wrong.setBackground(Color.lightGray);
    nextLessonItem();
  }

  public void selectedOpen() {
    if (debug)
      System.out.println("selected open");
    chooser = new LessonChooser(this, this, app, false, registered);
    chooser.setTitle("Open Vocab File...");
    chooser.setVisible(true);

  }

  protected void selectedLessonChooserOK() {
    // returned from dialog
    //toFront();
    chooser.setVisible(false);
    URL[] lessonURLs = chooser.getURLs();
    chooser.dispose();
    // If the user cancels then lessonURL could be set to null
    // therefore, only load a lesson if fileName is not null
    if (lessonURLs != null && lessonURLs[0] != null)
      if (loadLesson(lessonURLs)) {
	nextLessonItem();
      }
  }

  public boolean loadLesson(URL[] urls) {
    try {
      if (!Romaji2Kana.isLoaded()) {
	//load the romaji2kana conversion class' lookup table
	URL base = app.getCodeBase();
	String host = base.getHost();
	String protocol = base.getProtocol();
	int port =base.getPort();
	String origFile = base.getFile();
	String filePath = new String(origFile + "roma2kana.obj");
	URL url = new URL(protocol, host, port, filePath);
	Romaji2Kana.read(url.openStream());
      }

      for (int i=0; i<urls.length; i++) {
	InputStream in = urls[i].openStream();

	if (i==0) {
	  if (urls[i].getFile().endsWith(".kg") || urls[i].getFile().endsWith(".euc")) {
	    lesson = new Lesson(new InputStreamReader(in, "EUCJIS"));
	  }
	  else if (urls[i].getFile().endsWith(".jis")) {
	    lesson = new Lesson(new InputStreamReader(in, "JIS"));
	  }
	  else { // default is EUC
	    lesson = new Lesson(new InputStreamReader(in, "EUCJIS"));
	  }
	}
	else {
	  if (urls[i].getFile().endsWith(".kg") || urls[i].getFile().endsWith(".euc"))
	    lesson.appendLesson(new InputStreamReader(in, "EUCJIS"));
	  else if (urls[i].getFile().endsWith(".jis"))
	    lesson.appendLesson(new InputStreamReader(in, "JIS"));
	  else // default is EUC
	    lesson.appendLesson(new InputStreamReader(in, "EUCJIS"));
	}

	if (urls.length==1) {
	  fileName = urls[i].getFile();
	  StringTokenizer strTok = new StringTokenizer(fileName, "/");
	  while (strTok.hasMoreTokens())
	    fileName = strTok.nextToken();

	  lessonName = new String("Lesson " + fileName);
	  status.setText("Status: " + lessonName + " in progress (" + lesson.getSize() + ")");
	}
	else {
	  lessonName = new String("Multiple lessons");
	  status.setText("Status: " + lessonName + " in progress (" + lesson.getSize() + ")");
	}
      }
      commonPostOpen();
      return true;
    }
    catch(IOException ioe) {
      loadError();
      return false;
    }
  }

  public void selectedOpenWithLocalIO() {
    if (debug)
      System.out.println("selected open with local IO");
    FileDialog fileDialog;
    fileDialog = new FileDialog(this, "Open Vocab File...",FileDialog.LOAD);
    fileDialog.setDirectory("cards");
    fileDialog.setVisible(true);
    String fileName = fileDialog.getFile();
    String dir = fileDialog.getDirectory();
    // If the user cancels then fileName could be set to null
    // therefore, only load a lesson if fileName is not null
    if (fileName != null) {
      if (loadLesson(dir, fileName))
	nextLessonItem();
    }
  }

  public boolean loadLesson(String dir, String fileName) {
    try {

      if (!Romaji2Kana.isLoaded())
	//load the romaji2kana conversion class' lookup table
	Romaji2Kana.read(new FileInputStream("roma2kana.obj"));

      if (lesson != null) // save currently loaded lesson data
	lesson.save();
      FileInputStream in = new FileInputStream(dir+fileName);
      if (fileName.endsWith(".kg") || fileName.endsWith(".euc"))
	lesson = new LessonWithMemory(new InputStreamReader(in, "EUCJIS"),
				      "data/"+fileName+".obj");
      else if (fileName.endsWith(".jis"))
	lesson = new LessonWithMemory(new InputStreamReader(in, "JIS"),
				      "data/"+fileName+".obj");
      else // default is EUC
	lesson = new LessonWithMemory(new InputStreamReader(in, "EUCJIS"),
				      "data/"+fileName+".obj");
      this.fileName = fileName;
      
      lessonName = fileName;
      status.setText("Status: Lesson " + fileName + " in progress (" + lesson.getSize() + ")");
      numPresented = 0;
      commonPostOpen();
      return true;
    }
    catch(IOException ioe) {
      loadError();
      return false;
    }
  }

  private void loadError() {
    lesson = null;
    curItem = null;
    right.setEnabled(false);
    wrong.setEnabled(false);
    show.setEnabled(false);
    show.setBackground(Color.lightGray);
    restartMI.setEnabled(false);
    clearHistoryMI.setEnabled(false);
    openMI.setEnabled(true);
    status.setText("Status: Loading error!!!");
  }

  public void selectedMultiOpen() {
    if (debug)
      System.out.println("selected multi-open");
    String dir = "cards/";
    FileChooser chooser = new FileChooser(this, dir, true);
    chooser.setTitle("Open Vocab Files...");
    chooser.setVisible(true);
    toFront();
    String[] files = chooser.getFiles();
    // If the user cancels then files could be set to null
    // therefore, only load a lesson if fileName is not null
    if (files != null && files[0] != null)
      if (loadLesson(dir, files)) {
	nextLessonItem();
      }
  }

  public boolean loadLesson(String dir, String[] files) {
    try {

      if (!Romaji2Kana.isLoaded())
	//load the romaji2kana conversion class' lookup table
	Romaji2Kana.read(new FileInputStream("roma2kana.obj"));

      if (lesson != null) // save currently loaded lesson data
	lesson.save();
      for (int i=0; i<files.length; i++) {
	FileInputStream in = new FileInputStream(dir+files[i]);

	String lFileName = files[i];

	if (i==0) {
	  if (files[i].endsWith(".kg") || files[i].endsWith(".euc"))
	    lesson = new LessonWithMemory(new InputStreamReader(in, "EUCJIS"),
					  "data/"+lFileName+".obj");
	  else if (files[i].endsWith(".jis"))
	    lesson = new LessonWithMemory(new InputStreamReader(in, "JIS"),
					  "data/"+lFileName+".obj");
	  else // default is EUC
	    lesson = new LessonWithMemory(new InputStreamReader(in, "EUCJIS"),
					  "data/"+lFileName+".obj");
	}
	else {
	  if (files[i].endsWith(".kg") || files[i].endsWith(".euc"))
	    lesson.appendLesson(new InputStreamReader(in, "EUCJIS"),
				"data/"+lFileName+".obj");
	  else if (files[i].endsWith(".jis"))
	    lesson.appendLesson(new InputStreamReader(in, "JIS"),
				"data/"+lFileName+".obj");
	  else // default is EUC
	    lesson.appendLesson(new InputStreamReader(in, "EUCJIS"),
				"data/"+lFileName+".obj");
	}

	if (files.length==1) {
	  fileName = lFileName;
	  lessonName = new String("Lesson " + fileName);
	  status.setText("Status: " + lessonName + " in progress (" + lesson.getSize() + ")");
	}
	else {
	  lessonName = new String("Multiple lessons");
	  status.setText("Status: " + lessonName + " in progress (" + lesson.getSize() + ")");
	}
	numPresented = 0;
      }
      commonPostOpen();
      return true;
    }
    catch(IOException ioe) {
      loadError();
      return false;
    }
  }

  public void selectedPreferences() {
    if (debug)
      System.out.println("selected preferences");

    prefDialog = new Dialog(this, "Preferences", false);
    prefDialog.setLayout(new GridBagLayout());

    // create panel for item display checkboxes
    Panel displayPanel = new BorderedPanel(new GridBagLayout());
    Tools.constrain(displayPanel, new Label("Display:"), 0, 0, 1, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.WEST,
		    1.0, 0.25, 5, 5, 5, 5);
    Tools.constrain(displayPanel, yomiShowing, 0, 1, 1, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.CENTER,
		    1.0, 0.25, 5, 5, 5, 5);
    Tools.constrain(displayPanel, meaningShowing, 0, 2, 1, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.CENTER,
		    1.0, 0.25, 5, 5, 5, 5);
    Tools.constrain(displayPanel, kanjiShowing, 0, 3, 1, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.CENTER,
		    1.0, 0.25, 5, 5, 5, 5);

    Tools.constrain(prefDialog, new Label("Kanji Size"), 0, 0, 1, 1,
		    GridBagConstraints.NONE, GridBagConstraints.EAST,
		    0.5, 0.17, 5, 5, 5, 5);
    Tools.constrain(prefDialog, fontSizeChoice, 1, 0, 1, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.WEST,
		    0.5, 0.17, 5, 5, 5, 5);
    Tools.constrain(prefDialog, new Label("Lesson Mode"), 0, 1, 1, 1,
		    GridBagConstraints.NONE, GridBagConstraints.EAST,
		    0.5, 0.17, 5, 5, 5, 5);
    Tools.constrain(prefDialog, lessonModeChoice, 1, 1, 1, 1,
		    GridBagConstraints.BOTH, GridBagConstraints.WEST,
		    0.5, 0.17, 5, 5, 5, 5);
    Tools.constrain(prefDialog, displayPanel, 0, 2, 2, 4,
		    GridBagConstraints.BOTH, GridBagConstraints.CENTER,
		    1.0, 0.66, 5, 5, 5, 5);
    Tools.constrain(prefDialog, prefOK, 0, 6, 2, 1,
		    GridBagConstraints.NONE, GridBagConstraints.CENTER,
		    1.0, 0.17, 5, 5, 5, 5);
    prefDialog.pack();

    Rectangle bounds = this.getBounds();
    Rectangle abounds = prefDialog.getBounds();

    prefDialog.setLocation(bounds.x + (bounds.width - abounds.width)/ 2,
			   bounds.y + (bounds.height - abounds.height)/2);

    prefDialog.show();
  }

  protected void selectedPreferencesOK() {
    if (debug)
      System.out.println("selected preference's OK Button");
    prefDialog.dispose();
  }

  public void selectedExit() {
    if (debug)
      System.out.println("selected exit");
    if (lesson != null)
      lesson.save();
    this.setVisible(false);
    this.dispose();
    if (app == null)
      System.exit(0);
  }

  public void selectedRestart() {
    if (debug)
      System.out.println("selected restart");
    if (lesson != null) {
      lesson.reset();
      if (!registered || app != null || lessonModeChoice.getSelectedIndex() != 3)
	results.setText("Results: 0/0 (" + lesson.getCurSize() + ")");
      else
	results.setText("Results: 0/0");
      status.setText("Status: " + lessonName + " in progress (" + lesson.getSize() + ")");
      numPresented = 0;
      nextLessonItem();
    }
  }

  public void selectedClearHistory() {
    if (debug)
      System.out.println("selected clear history");
    if (lesson != null)
      lesson.clearHistory();
  }

  public void selectedNormalLesson() {
    if (debug)
      System.out.println("selected normal lesson ");
    lesson.setReviewMode(false);
    if (lesson != null) {
      lesson.hardReset();
      results.setText("Results: 0/0 (" + lesson.getCurSize() + ")");
      status.setText("Status: " + lessonName + " in progress (" + lesson.getSize() + ")");
      nextLessonItem();
    }
  }

  public void selectedQuiz(int count) {
    if (debug)
      System.out.println("selected quiz " + count);
    if (lesson != null) {
      lesson.setReviewMode(false);
      lesson.randomSubset(count);
      results.setText("Results: 0/0 (" + lesson.getCurSize() + ")");
      status.setText("Status: " + lessonName + " in progress (" + lesson.getSize() + ")");
      nextLessonItem();
    }
  }

  public void selectedReviewMode() {
    if (debug)
      System.out.println("selected review mode");
    if (lesson != null) {
      lesson.hardReset();
      lesson.setReviewMode(true);
      results.setText("Results: 0/0");
      status.setText("Status: " + lessonName + " in progress (" + lesson.getSize() + ")");
      nextLessonItem();
    }
  }

  public void setKanjiMaxSize(int maxSize) {
    if (maxSize != 0)
      kanji.setMaxSize(maxSize);
    else
      kanji.setMaxSize(FontCanvas.DEFAULT_MAXSIZE);
    kanji.repaint();
  }

  public void selectedYomiShowing() {
    yomi.setVisible(yomiShowing.getState());
  }

  public void selectedMeaningShowing() {
    meaning.setVisible(meaningShowing.getState());
  }

  public void selectedKanjiShowing() {
    kanji.setVisible(kanjiShowing.getState());
  }

  public void selectedAbout() {
    if (debug)
      System.out.println("selected about");
    AboutBox about = new AboutBox(this, "KanjiFlash " + version, copyright,
				  registered?"Registered - Thank you" : "Unregistered",
				  "email: mramsey@bpa.arizona.edu");
    about.setVisible(true);
  }

  protected void loadTextCanvases() {
    if (!useBitMaps) {
      meaning = new FontCanvas(FontCanvas.LEFT, 12);
      meaning.setFont(new Font("sansserif", Font.PLAIN, 12));
      meaning.setBackground(Color.white);
      meaning.setForeground(Color.black);

      yomi = new FontCanvas(FontCanvas.LEFT, 18);
      yomi.setFont(new Font("serif", Font.PLAIN, 18));
      yomi.setBackground(Color.white);
      yomi.setForeground(Color.black);
    
      kanji = new FontCanvas(FontCanvas.CENTER, fontSize);
      kanji.setFont(new Font("serif", Font.PLAIN, FontCanvas.DEFAULT_MAXSIZE));
      kanji.setBackground(Color.white);
      kanji.setForeground(Color.red);
      kanji.setText("\u6f22\u5b57Flash");
    }
    else {

      largeFont = null;
      smallFont = null;
      InputStream largeIn = null;
      InputStream smallIn = null;

      try {
	if (app == null) {
	  // not an applet so can load from a file w/o violating security
	  FileInputStream largeFileIn = 
                      new FileInputStream((new RandomAccessFile("k48x48.gz","r")).getFD());
	  FileInputStream smallFileIn = 
                      new FileInputStream((new RandomAccessFile("k16x16.gz","r")).getFD());
	  largeIn = new GZIPInputStream(largeFileIn);
	  smallIn = new GZIPInputStream(smallFileIn);
	  largeFont=new JISBitMapFont(48, 48, largeIn, 2247040,   // k48x48.f00
				      8, 64, true, false);
	  //largeFont=new JISBitMapFont(24, 24, largeIn, 561808,  // k24x24.f00
	  //			      8, 64, true, false);
	  //smallFont=new JISBitMapFont(24, 24, smallIn, 562032,  // asiya.fnt
	  //				 8, 0, true, true);
	  smallFont=new JISBitMapFont(16, 16, smallIn, 220160,    // k16x16.f00
				      8, 64, false, false);
	}
	else {
	  // applet so load via URL
	  URL base = app.getCodeBase();
	  String host = base.getHost();
	  String protocol = base.getProtocol();
	  int port =base.getPort();
	  String origFile = base.getFile();
	  String largePath = null;
	  String smallPath = null;
	  URL largeUrl = null;
	  URL smallUrl = null;

	  smallPath = new String(origFile + "k16x16.gz");
	  smallUrl = new URL(protocol, host, port, smallPath);
	  smallIn = new GZIPInputStream(smallUrl.openStream());
	  smallFont=new JISBitMapFont(16, 16, smallIn, 220160,    // k16x16.f00
				      8, 64, false, false);

	  String param = app.getParameter("fontSize");
	  
	  if (param != null && param.equals("large")) {
	    largePath = new String(origFile + "k48x48.gz");
	    largeUrl = new URL(protocol, host, port, largePath);
	    largeIn = new GZIPInputStream(largeUrl.openStream());
	    largeFont=new JISBitMapFont(48, 48, largeIn, 2247040,   // k48x48.f00
					8, 64, true, false);
	  }
	  else if (param != null && param.equals("medium")) {
	    largePath = new String(origFile + "k24x24.gz");
	    largeUrl = new URL(protocol, host, port, largePath);
	    largeIn = new GZIPInputStream(largeUrl.openStream());
	    largeFont=new JISBitMapFont(24, 24, largeIn, 561808,  // k24x24.f00
					8, 64, true, false);
	    //largeFont=new JISBitMapFont(24, 24, smallIn, 562032,  // asiya.fnt
	    //				8, 0, true, true);
	  }
	  else { // small font
	    largeFont=smallFont;
	  }
	}

      }
      catch (IOException ioe) {
	Tools.perr(ioe);
      }
      
      BitMapTextCanvas lbm = new BitMapTextCanvas(largeFont, 2, -1);
      lbm.setJustification(BitMapTextCanvas.CENTER);
      kanji = lbm;
      kanji.setBackground(Color.white);
      kanji.setForeground(Color.red);
      kanji.setText("\u6f22\u5b57\u30d5\u30e9\u30b7\u30e5");

      yomi = new BitMapTextCanvas(smallFont, 2, 10);
      yomi.setBackground(Color.white);
      yomi.setForeground(Color.black);

      meaning = new FontCanvas(FontCanvas.LEFT, 12);
      meaning.setFont(new Font("sansserif", Font.PLAIN, 12));
      meaning.setBackground(Color.white);
      meaning.setForeground(Color.black);
    }
  }

  public static void main(String[] args) {
    boolean registered = true;
    boolean useBitMaps = false;
    for (int i=0; i<args.length; i++) {
      if (args[i].equals("-unreg"))
	registered = false;
      if (args[i].equals("-bitmap"))
	useBitMaps = true;
    }
    new KanjiFlash(null, registered, useBitMaps);
  }
}
