/* This code is copyrighted (c) 1997 by Marshall Ramsey */
import java.util.*;
import java.io.*;

public class LessonWithMemory extends Lesson {

  static final double MINODDS = 1.0;
  static final double INITIALODDS = 1.0;
  static final double RIGHTLEARNRATE = -1.0;
  static final double WRONGLEARNRATE = 6.0;

  RouletteWheel mainWheel;
  Vector wheelInfo;
  boolean reviewMode;
  LessonItem prevItem;

  public LessonWithMemory(InputStreamReader in, String wheelFileName) {
    super(in);
    initialize(wheelFileName);
  }

  public LessonWithMemory(String input, String wheelFileName) {
    super(input);
    initialize(wheelFileName);
  }

  private void initialize(String wheelFileName) {
    reviewMode = false;
    wheelInfo = new Vector();
    prevItem = null;
    try {
      FileInputStream fin = new FileInputStream(wheelFileName);
      ObjectInputStream oin = new ObjectInputStream(fin);
      mainWheel = (RouletteWheel) oin.readObject();
      fin.close();
      if (mainWheel.size() != origList.size()) { // inconsistent RouletteWheel
	mainWheel = new RouletteWheel(MINODDS, INITIALODDS);
	for (int i=0; i<origList.size(); i++)
	  mainWheel.addElement(); // add element with default odds
      }
      wheelInfo.addElement(new WheelElement(mainWheel.size(), wheelFileName));
    }
    catch(IOException ioe) {
      mainWheel = new RouletteWheel(MINODDS, INITIALODDS);
      for (int i=0; i<items.size(); i++)
	mainWheel.addElement(); // add element with default odds
      wheelInfo.addElement(new WheelElement(mainWheel.size(), wheelFileName));
    }
    catch(ClassNotFoundException cnfe) {
      mainWheel = new RouletteWheel(MINODDS, INITIALODDS);
      for (int i=0; i<items.size(); i++)
	mainWheel.addElement(); // add element with default odds
      wheelInfo.addElement(new WheelElement(mainWheel.size(), wheelFileName));
    }
  }

  public void appendLesson(InputStreamReader in, String wheelFileName) {
    int startSize = getSize();
    appendLesson(in);
    append(wheelFileName, getSize() - startSize);
  }

  public void appendLesson(String string, String wheelFileName) {
    int startSize = getSize();
    appendLesson(string);
    append(wheelFileName, getSize() - startSize);
  }

  private void append(String wheelFileName, int numItemsAdded) {
    RouletteWheel temp = null;
    try {
      FileInputStream fin = new FileInputStream(wheelFileName);
      ObjectInputStream oin = new ObjectInputStream(fin);
      temp = (RouletteWheel) oin.readObject();
      fin.close();
      if (temp.size() != numItemsAdded) { // inconsistent RouletteWheel
	temp = new RouletteWheel(MINODDS, INITIALODDS);
	for (int i=0; i<numItemsAdded; i++)
	  temp.addElement(); // add element with default odds
      }
      wheelInfo.addElement(new WheelElement(temp.size(), wheelFileName));
      mainWheel.addWheel(temp);
    }
    catch(IOException ioe) {
      temp = new RouletteWheel(MINODDS, INITIALODDS);
      for (int i=0; i<numItemsAdded; i++)
	temp.addElement(); // add element with default odds
      wheelInfo.addElement(new WheelElement(temp.size(), wheelFileName));
      mainWheel.addWheel(temp);
    }
    catch(ClassNotFoundException cnfe) {
      temp = new RouletteWheel(MINODDS, INITIALODDS);
      for (int i=0; i<numItemsAdded; i++)
	temp.addElement(); // add element with default odds
      wheelInfo.addElement(new WheelElement(temp.size(), wheelFileName));
      mainWheel.addWheel(temp);
    }
  }

  public LessonItem nextItem() {
    try {
      if (reviewMode) {
	LessonItem curItem = null;
	if (origList.size() > 1)
	  while (prevItem == (curItem = (LessonItem) origList.elementAt(mainWheel.getRandomElement())));
	prevItem = curItem;
	return curItem;
      }
      else
	return super.nextItem();
    }
    catch (ArrayIndexOutOfBoundsException aioobe) {
      System.err.println("Error in data file. Recreating file");
      clearHistory();
      if (reviewMode)
	return (LessonItem) origList.elementAt(mainWheel.getRandomElement());
      else
	return super.nextItem();
    }
  }

  public void markRight(LessonItem rightItem) {
    super.markRight(rightItem);
    mainWheel.modifyOdds(getItemNum(rightItem), RIGHTLEARNRATE);
  }

  public void markWrong(LessonItem wrongItem) {
    super.markWrong(wrongItem);
    mainWheel.modifyOdds(getItemNum(wrongItem), WRONGLEARNRATE);
  }

  public void setReviewMode(boolean mode) { reviewMode = mode; }

  private int getItemNum(LessonItem item) {
    for (int i=0; i<origList.size(); i++) {
      LessonItem temp = (LessonItem) origList.elementAt(i);
      if (temp == item)
	return i;
    }
    return -1; // error flag
  }

  public void clearHistory() {
    mainWheel = new RouletteWheel(MINODDS, INITIALODDS);
    for (int i=0; i<origList.size(); i++)
      mainWheel.addElement(); // add element with default odds
  }

  public void save() {
    try {
      int i=0;
      for (int j=0; j<wheelInfo.size(); j++) {
	WheelElement temp = (WheelElement)wheelInfo.elementAt(j);
	RouletteWheel wheel = new RouletteWheel(temp.size);
	for (int k=i; k<i+temp.size; k++)
	  wheel.addElement(mainWheel.getOddsForElementAt(k));

	FileOutputStream fout = new FileOutputStream(temp.wheelFileName);
	ObjectOutputStream out = new ObjectOutputStream(fout);
	out.writeObject(wheel);
	fout.close();
      }
    }
    catch(IOException ioe) {
      Tools.perr(ioe);
    }
  }

}
