/* This code is copyrighted (c) 1997 by Marshall Ramsey */
import java.awt.*;
import java.util.*;
import java.awt.event.*;

public class ScribbleCanvas extends Canvas implements MouseListener, MouseMotionListener {

  private int last_x = 0;
  private int last_y = 0;
  Vector points;
  Vector curArc;
    
  public ScribbleCanvas() {
    // Set the default background & foreground color
    this.setBackground(Color.white);
    this.setForeground(Color.black);
    points = new Vector(100);
    curArc = new Vector();
    addMouseListener(this);
    addMouseMotionListener(this);
  }
    
  // Called when the user clicks the mouse to start a scribble
  public void mousePressed(MouseEvent e) {
    try {
      int x = e.getX();
      int y = e.getY();
      last_x = x;
      last_y = y;
      curArc = new Vector();
      points.addElement(curArc);
      curArc.addElement(new ColorPoint(x, y, getForeground()));
    }
    catch(NoClassDefFoundError error) {
    }
  }
    
  // Called when the user scribbles with the mouse button down
  public void mouseDragged(MouseEvent e) {
    int x = e.getX();
    int y = e.getY();
    curArc.addElement(new ColorPoint(x,y,getForeground()));
    Graphics g = this.getGraphics();
    connectPoints(g, last_x, last_y, x, y);
    last_x = x;
    last_y = y;
  }

  public void repaint() {
    update(getGraphics());
  }

  public void update(Graphics g) {
    paint(g);
  }

  public void paint(Graphics g) {
    for (int i=0; i<points.size(); i++) {
      Vector arc = (Vector) points.elementAt(i);
      try {
	ColorPoint prevPt = (ColorPoint) arc.elementAt(0);
	ColorPoint curPt;
	for (int j=1; true; j++) {
	  curPt = (ColorPoint) arc.elementAt(j);
	  g.setColor(curPt.color);
	  connectPoints(g, prevPt.x, prevPt.y, curPt.x, curPt.y);
	  prevPt = curPt;
	}
      }
      catch(ArrayIndexOutOfBoundsException aioobe) {
	//used to exit the for loop so do nothing
      }
    }

    drawBorder(g);
  }

  public void clear() {
    //    Rectangle r = bounds();
    Graphics g = getGraphics();
    g.setColor(getBackground());
    //    g.fillRect(r.x, r.y, r.width, r.height);
    Dimension d = getSize();
    points.setSize(0);
    curArc = new Vector();
    g.fillRect(0, 0, d.width, d.height);
	drawBorder(g);
  }

  private void drawBorder(Graphics g) {
    //draw border
    Dimension d = getSize();
    g.setColor(Color.gray);
    g.drawLine(0, 0, d.width-1, 0);
    g.drawLine(0, 1, 0, d.height-1);
    g.setColor(Color.black);
    g.drawLine(1, 1, d.width-2, 1);
    g.drawLine(1, 2, 1, d.height-2);
    g.setColor(Color.lightGray);
    g.drawLine(d.width-2, 1, d.width-2, d.height-2);
    g.drawLine(1, d.height-2, d.width-2, d.height-2);
    g.setColor(Color.white);
    g.drawLine(d.width-1, 0, d.width-1, d.height-1);
    g.drawLine(0, d.height-1, d.width-1, d.height-1);
  }

  private void connectPoints(Graphics g, int last_x, int last_y, int x, int y) {
    try {
      int xPts[] = new int[6];
      int yPts[] = new int[6];

      /* slanted line */
      /*
      xPts[0] = last_x;
      xPts[1] = last_x+5;
      xPts[2] = x+5;
      xPts[3] = x;
	
      yPts[0] = last_y;
      yPts[1] = last_y+5;
      yPts[2] = y+6;
      yPts[3] = y;

      */

      /*
      int x_width = 5;
      int y_width = 5;

      xPts[0] = last_x;
      xPts[1] = last_x+x_width;
      xPts[2] = last_x+x_width;
      xPts[3] = x+x_width;
      xPts[4] = x;
      xPts[5] = x;
	
      yPts[0] = last_y;
      yPts[1] = last_y;
      yPts[2] = last_y+y_width;
      yPts[3] = y+y_width;
      yPts[4] = y+y_width;
      yPts[5] = y;

      g.fillPolygon(xPts, yPts, xPts.length);
      */

      /* solid block line */
      g.drawLine(last_x, last_y, x, y);
      g.drawLine(last_x, last_y+1, x, y+1);
      g.drawLine(last_x, last_y+2, x, y+2);
      g.drawLine(last_x+1, last_y, x+1, y);
      g.drawLine(last_x+2, last_y, x+2, y);
      g.drawLine(last_x+3, last_y, x+3, y);

    }
    catch(OutOfMemoryError oome) {
      System.gc();
    }
  }

  private double slope(int x2, int y2, int x1, int y1) {
    return ((double)(y2 - y1))/((double)(x2 - x1));
  }

  // Unhandled MouseListener and MouseMotionListener functions
  public void mouseClicked(MouseEvent e) {}
  public void mouseEntered(MouseEvent e) {}
  public void mouseExited(MouseEvent e) {}
  public void mouseReleased(MouseEvent e) {}
  public void mouseMoved(MouseEvent e) {}
}
